package com.gxzc.zen;

import com.gxzc.zen.orm.DynamicDataSource;
import com.gxzc.zen.orm.DynamicMultipleDataSource;
import kotlin.TypeCastException;
import kotlin.jvm.internal.Intrinsics;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.reflect.MethodSignature;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Objects;

@Aspect
@Order(-1)
@Component
public class A {

    private static final Logger logger = LoggerFactory.getLogger(A.class);

//    @Before("execution(* com.gxzc.zen.api.sys..*Service.*(..))")
//    public void sysService(@NotNull JoinPoint joinPoint) {
//        setDataSource(joinPoint, "sysDataSource");
//    }

    @Before("execution(* com.baomidou.mybatisplus..*Service.*(..)) || execution(* com.baomidou.mybatisplus..*Mapper.*(..)) || execution(* com.gxzc.zen.api..*Service.*(..))")
    public void dataSourceSys2(@NotNull JoinPoint joinPoint) {
        Object o = joinPoint.getTarget();

        String packageName = null;
        try {
            Field h = o.getClass().getSuperclass().getDeclaredField("h");
            h.setAccessible(true);
            Object mapperProxy = h.get(o);

            Field mapperInterface = mapperProxy.getClass().getDeclaredField("mapperInterface");
            mapperInterface.setAccessible(true);

            Class<?> clazz = (Class<?>) mapperInterface.get(mapperProxy);

            packageName = clazz.getPackage().getName();
        } catch (Exception e) {
            logger.debug("get packageName of MapperProxy error, ignore this info if nothing happened!");
        } finally {
            if (StringUtils.isBlank(packageName)) {
                logger.debug("get packageName of target...");
                packageName = joinPoint.getTarget().getClass().getPackage().getName();
            }
        }

        matchDataSource(joinPoint, packageName);
    }

//    @Before("execution(* com.gxzc.zen.api.rec..*Service.*(..))")
//    public void recService(@NotNull JoinPoint joinPoint) {
//        setDataSource(joinPoint, "recDataSource");
//    }

    private void matchDataSource(JoinPoint joinPoint, String packageName) {
        if (packageName.contains("com.gxzc.zen.api.sys")) {
            setDataSource(joinPoint, "sysDataSource");
        } else if (packageName.contains("com.gxzc.zen.api.rec")) {
            setDataSource(joinPoint, "recDataSource");
        }
    }

    private void setDataSource(JoinPoint joinPoint, String defaultKey) {
        DynamicDataSource dynamicDataSource = this.currentMethod(joinPoint).getAnnotation(DynamicDataSource.class);
        if (Objects.isNull(dynamicDataSource)) {
            DynamicMultipleDataSource.Companion.setDataSource(defaultKey);
        } else {
            DynamicMultipleDataSource.Companion.setDataSource(dynamicDataSource.value());
        }
    }

    private Method currentMethod(JoinPoint joinPoint) {
        Signature var10000 = joinPoint.getSignature();
        if (var10000 == null) {
            throw new TypeCastException("null cannot be cast to non-null type org.aspectj.lang.reflect.MethodSignature");
        } else {
            Method var2 = ((MethodSignature) var10000).getMethod();
            Intrinsics.checkExpressionValueIsNotNull(var2, "(joinPoint.signature as MethodSignature).method");
            return var2;
        }
    }
}
