package com.gxzc.zen.orm

import org.aspectj.lang.JoinPoint
import org.aspectj.lang.annotation.Aspect
import org.aspectj.lang.annotation.Before
import org.aspectj.lang.reflect.MethodSignature
import org.springframework.core.annotation.Order
import java.lang.reflect.Method
import java.util.*

/**
 *
 * @author NorthLan at 2018/1/29
 */
@Aspect
@Order(-1)
class DataSourceSwitchAspect {

    @Before("execution(* com.gxzc.zen.api.sys.*.*Service.*(..))")
    fun dataSourceSys(joinPoint: JoinPoint) {
        setDataSource(joinPoint, MultipleDataSourceConfig.DS_KEY_SYS)
    }

    @Before("execution(* com.gxzc.zen.api.rec.*.*Service.*(..))")
    fun dataSourceRec(joinPoint: JoinPoint) {
        setDataSource(joinPoint, MultipleDataSourceConfig.DS_KEY_REC)
    }

    /**
     * 设置数据源key
     */
    private fun setDataSource(joinPoint: JoinPoint, defaultKey: String) {
        val dynamicDataSource = currentMethod(joinPoint).getAnnotation(DynamicDataSource::class.java)
        if (Objects.isNull(dynamicDataSource)) {
            DynamicMultipleDataSource.setDataSource(defaultKey)
            return
        }
        DynamicMultipleDataSource.setDataSource(dynamicDataSource.value)
    }

    private fun currentMethod(joinPoint: JoinPoint): Method {
        return (joinPoint.signature as MethodSignature).method
    }
}