package com.gxzc.zen.orm

import com.alibaba.druid.pool.DruidDataSource
import com.baomidou.mybatisplus.spring.MybatisSqlSessionFactoryBean
import com.baomidou.mybatisplus.spring.boot.starter.MybatisPlusProperties
import com.baomidou.mybatisplus.spring.boot.starter.SpringBootVFS
import org.mybatis.spring.annotation.MapperScan
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.boot.context.properties.ConfigurationProperties
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Primary
import org.springframework.core.io.support.PathMatchingResourcePatternResolver
import org.springframework.jdbc.datasource.DataSourceTransactionManager
import javax.sql.DataSource

/**
 *
 * @author NorthLan at 2018/1/29
 */
@Configuration
@EnableConfigurationProperties(MybatisPlusProperties::class)
@MapperScan("com.gxzc.zen.api.*.mapper")
class MultipleDataSourceConfig {
    companion object {
        const val DS_KEY_SYS = "sysDataSource"
        const val DS_KEY_REC = "recDataSource"
    }

    @Autowired
    private lateinit var properties: MybatisPlusProperties

    private val mapperLocation: String = "classpath:/mapping/*/*.xml"


    @Bean(DS_KEY_SYS)
    @ConfigurationProperties(prefix = "datasource.sys")
    @Primary
    fun dataSourceSys(): DataSource {
        return DruidDataSource()
    }

    @Bean(DS_KEY_REC)
    @ConfigurationProperties(prefix = "datasource.rec")
    fun dataSourceRec(): DataSource {
        return DruidDataSource()
    }

    @Bean
    fun multipleDataSource(@Qualifier(DS_KEY_SYS) dataSourceSys: DataSource,
                           @Qualifier(DS_KEY_REC) dataSourceRec: DataSource): DynamicMultipleDataSource {
        return DynamicMultipleDataSource().also {
            it.setTargetDataSources(mapOf(Pair(DS_KEY_SYS, dataSourceSys), Pair(DS_KEY_REC, dataSourceRec)))
            it.setDefaultTargetDataSource(dataSourceSys)
        }
    }

    @Bean
    fun transactionManager(dynamicMultipleDataSource: DynamicMultipleDataSource): DataSourceTransactionManager {
        return DataSourceTransactionManager(dynamicMultipleDataSource)
    }

    @Bean
    @Primary
    fun mybatisSqlSessionFactoryBean(dynamicMultipleDataSource: DynamicMultipleDataSource): MybatisSqlSessionFactoryBean {
        return MybatisSqlSessionFactoryBean().also {
            it.setDataSource(dynamicMultipleDataSource)
            it.vfs = SpringBootVFS::class.java
            it.setConfiguration(properties.configuration)
            it.setGlobalConfig(properties.globalConfig?.convertGlobalConfiguration())
            it.setMapperLocations(PathMatchingResourcePatternResolver().getResources(mapperLocation))
        }
    }

//    @Bean
//    fun sqlSessionFactory(dynamicMultipleDataSource: DynamicMultipleDataSource): SqlSessionFactory {
//        return SqlSessionFactoryBean().let {
//            it.setDataSource(dynamicMultipleDataSource)
//            it.`object`
//        }
//    }
}